package com.supervision.knowsub.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.entity.vo.knowledge.KnowledgeLinkVO;
import com.supervision.knowsub.entity.vo.knowledge.ModifyKnowledgeReqVO;
import com.supervision.knowsub.entity.vo.knowledge.SaveKnowledgeReqVO;
import com.supervision.knowsub.enums.StatusEnum;
import com.supervision.knowsub.exception.BusinessException;
import com.supervision.knowsub.model.*;
import com.supervision.knowsub.service.*;
import com.supervision.knowsub.util.UserUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class KnowledgeManageServiceImpl implements KnowledgeManageService {

    private final KnowledgeService knowledgeService;

    private final KnowledgeContextService knowledgeContextService;

    private final KnowledgeBaseInfoService knowledgeBaseInfoService;

    private final KnowledgeAttachmentService knowledgeAttachmentService;

    private final KnowledgeLinkService knowledgeLinkService;

    private final FileService fileService;


    @Override
    public IPage<Knowledge> queryKnowledgePage(Integer status, String title, String publishDeptName, Integer pageNum, Integer pageSize) {
        String userId = UserUtil.getUser().getId();
        return knowledgeService.queryKnowledgePage(status, title, publishDeptName, userId, pageNum, pageSize);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveKnowledge(SaveKnowledgeReqVO reqVO) {
        var context = new KnowledgeContext();
        context.setContext(reqVO.getContent());
        knowledgeContextService.save(context);

        Knowledge knowledge = new Knowledge();

        knowledge.setTitle(reqVO.getTitle());
        knowledge.setContentId(context.getId());
        // TODO 这里目前子库设置为空
        knowledge.setBaseId(null);
        knowledge.setPublishDept(reqVO.getPublishDept());
        // 目前只有人工添加
        knowledge.setKnowledgeFrom(1);
        knowledge.setStatus(reqVO.getOperate() == 1  ? StatusEnum.DRAFT.getStatus() : StatusEnum.WAIT_APPROVAL.getStatus());
        knowledge.setSubmittedDeptId(reqVO.getSubmittedDeptId());
        if (1 == reqVO.getOperate()) {
            knowledge.setDraftBelongUserId(reqVO.getUserId());
        }
        knowledgeService.save(knowledge);
        // 保存知识的基本信息
        KnowledgeBaseInfo baseInfo = reqVO.getKnowledgeBaseInfo();
        baseInfo.setContextId(context.getId());
        knowledgeBaseInfoService.save(baseInfo);
        // 附件以及URL没添加
        for (String fileId : reqVO.getFileIdList()) {
            KnowledgeAttachment attachment = new KnowledgeAttachment();
            attachment.setKnowledgeId(knowledge.getId());
            attachment.setFileId(fileId);
            knowledgeAttachmentService.save(attachment);
        }
        for (KnowledgeLinkVO knowLedgeLinkVO : reqVO.getLinkList()) {
            KnowledgeLink knowledgeLink = new KnowledgeLink();
            knowledgeLink.setLinkName(knowLedgeLinkVO.getLinkName());
            knowledgeLink.setLinkUrl(knowLedgeLinkVO.getLinkUrl());
            knowledgeLinkService.save(knowledgeLink);
        }
        // 这里需要进行判断,如果状态为2,说明要走流程
        if (1 != reqVO.getOperate()) {
            // TODO 这里需要走流程
        }


    }

    /**
     *
     * @param reqVO
     */
    @Override
    public void updateKnowledge(ModifyKnowledgeReqVO reqVO) {
        Knowledge knowledge = knowledgeService.getOptById(reqVO.getKnowledgeId()).orElseThrow(() -> new BusinessException("未找到知识"));
        // 首先校验状态,草稿/已撤回/已失效/驳回,才能修改
        Set<Integer> canModifyStatusSet = Set.of(StatusEnum.DRAFT.getStatus(), StatusEnum.RECALL.getStatus(), StatusEnum.INVALID.getStatus(),StatusEnum.REJECT.getStatus());
        if (!canModifyStatusSet.contains(knowledge.getStatus())) {
            throw new BusinessException("当前状态不支持修改");
        }
        // 找到对应的content
        KnowledgeContext context = knowledgeContextService.getOptById(knowledge.getContentId()).orElse(KnowledgeContext.builder().id(reqVO.getKnowledgeId()).build());
        context.updateById();

        // 然后更新
        knowledge.setTitle(reqVO.getTitle());
        // TODO 这里目前子库设置为空
        knowledge.setPublishDept(reqVO.getPublishDept());
        knowledge.setSubmittedDeptId(reqVO.getSubmittedDeptId());
        knowledge.setStatus(reqVO.getOperate() == 1  ? StatusEnum.DRAFT.getStatus() : StatusEnum.WAIT_APPROVAL.getStatus());

        // 如果保存为草稿
        if (1 == reqVO.getOperate()) {
            knowledge.setDraftBelongUserId(reqVO.getUserId());
        }
        knowledgeService.updateById(knowledge);
        // 更新基础信息
        KnowledgeBaseInfo knowledgeBaseInfo = reqVO.getKnowledgeBaseInfo();
        Assert.notBlank(knowledgeBaseInfo.getKnowledgeId(),"知识ID不能为空");
        // 这里需要用SQL进行更新,不能用封装的updateById,因为前端某些字段可能变为空,用updateById不能让那个想为空的变成空
        knowledgeBaseInfoService.lambdaUpdate()
                .set(KnowledgeBaseInfo::getPolicyType,knowledgeBaseInfo.getPolicyType())
                .set(KnowledgeBaseInfo::getTerritory,knowledgeBaseInfo.getTerritory())
                .set(KnowledgeBaseInfo::getPublishScope,knowledgeBaseInfo.getPublishScope())
                .set(KnowledgeBaseInfo::getKnowledgeTag,knowledgeBaseInfo.getKnowledgeTag())
                .set(KnowledgeBaseInfo::getAutoLoseEffect,knowledgeBaseInfo.getAutoLoseEffect())
                .set(KnowledgeBaseInfo::getExecTimeBegin,knowledgeBaseInfo.getExecTimeBegin())
                .set(KnowledgeBaseInfo::getExecTimeEnd,knowledgeBaseInfo.getExecTimeEnd())
                .set(KnowledgeBaseInfo::getTimeliness,knowledgeBaseInfo.getTimeliness())
                .set(KnowledgeBaseInfo::getPublishDate,knowledgeBaseInfo.getPublishDate())
                .eq(KnowledgeBaseInfo::getKnowledgeId,knowledgeBaseInfo.getKnowledgeId()).update();
        // 有可能更新附件
        updateKnowledgeFile(reqVO, knowledge);
        // 有可能更新URL
        updateKnowledgeLink(reqVO, knowledge);
        // 这里需要进行判断,如果状态为2,说明要走流程
        if (1 != reqVO.getOperate()) {
            // TODO 这里需要走流程

        }
    }

    private void updateKnowledgeLink(ModifyKnowledgeReqVO reqVO, Knowledge knowledge) {
        // 直接删除原来的
        knowledgeLinkService.lambdaUpdate().eq(KnowledgeLink::getKnowledgeId,knowledge.getId()).remove();
        for (KnowledgeLinkVO knowLedgeLinkVO : reqVO.getLinkList()) {
            KnowledgeLink knowledgeLink = new KnowledgeLink();
            knowledgeLink.setLinkName(knowLedgeLinkVO.getLinkName());
            knowledgeLink.setLinkUrl(knowLedgeLinkVO.getLinkUrl());
            knowledgeLinkService.save(knowledgeLink);
        }

    }

    private void updateKnowledgeFile(ModifyKnowledgeReqVO reqVO, Knowledge knowledge) {
        // 获取数据库中的附件ID
        List<KnowledgeAttachment> oldFileList = knowledgeAttachmentService.lambdaQuery().eq(KnowledgeAttachment::getKnowledgeId, knowledge.getId()).list();
        if (CollUtil.isEmpty(reqVO.getFileIdList())){
            if (CollUtil.isNotEmpty(oldFileList)){
                knowledgeAttachmentService.lambdaUpdate().eq(KnowledgeAttachment::getKnowledgeId,knowledge.getId()).remove();
                // 删除文件
                for (KnowledgeAttachment knowledgeAttachment : oldFileList) {
                    fileService.deleteFileById(knowledgeAttachment.getFileId());
                }
            }

        }else {
            List<String> newFileIdList = reqVO.getFileIdList();
            // 遍历旧的附件,如果新的附件列表中不包含,则删除
            for (KnowledgeAttachment oldFile : oldFileList) {
                if (!newFileIdList.contains(oldFile.getId())){
                    knowledgeAttachmentService.lambdaUpdate().eq(KnowledgeAttachment::getFileId,oldFile.getId()).remove();
                    fileService.deleteFileById(oldFile.getFileId());
                }
            }
            List<String> oldFileIdList = oldFileList.stream().map(KnowledgeAttachment::getFileId).toList();
            // 再添加新的附件
            for (String fileId : reqVO.getFileIdList()) {
                if (!oldFileIdList.contains(fileId)){
                    KnowledgeAttachment attachment = new KnowledgeAttachment();
                    attachment.setKnowledgeId(knowledge.getId());
                    attachment.setFileId(fileId);
                    knowledgeAttachmentService.save(attachment);
                }

            }
        }

    }


}
